#if defined(__linux__)

#if defined(CROSS)
/* To avoid compile error by differ of return value of read()/write() */
#define _READ_WRITE_RETURN_TYPE _ssize_t
#endif
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include <nllibc.h>

int _set_errno(int retval)
{
  if ((retval < 0) && (retval > -4096)) {
    errno = -retval;
    retval = -1;
  }
  return retval;
}

void ___exit(int status);
ssize_t __read(int fd, void *buf, size_t nbytes);
ssize_t __write(int fd, const void *buf, size_t nbytes);
int __open(const char *path, int flags, mode_t mode);
int __close(int fd);
int __ioctl(int fd, unsigned long request, unsigned long args);

void _exit(int status)
{
  ___exit(status);
  while (1) /* to avoid noreturn warning */
    ;
}

ssize_t read(int fd, void *buf, size_t nbytes)
{
  return _set_errno(__read(fd, buf, nbytes));
}

ssize_t write(int fd, const void *buf, size_t nbytes)
{
  return _set_errno(__write(fd, buf, nbytes));
}

static int _va_open(const char *path, int flags, va_list ap)
{
  unsigned long arg;
  arg = va_arg(ap, unsigned long);
  return __open(path, flags, arg);
}

int open(const char *path, int flags, ...)
{
  va_list arg;
  int r;

  va_start(arg, flags);
  r = _va_open(path, flags, arg);
  va_end(arg);

  return _set_errno(r);
}

int close(int fd)
{
  return _set_errno(__close(fd));
}

static int _va_ioctl(int fd, unsigned long request, va_list ap)
{
  unsigned long arg;
  arg = va_arg(ap, unsigned long);
  return __ioctl(fd, request, arg);
}

int ioctl(int fd, unsigned long request, ...)
{
  va_list arg;
  int r;

  va_start(arg, request);
  r = _va_ioctl(fd, request, arg);
  va_end(arg);

  return _set_errno(r);
}

#endif
